/***********************************************************************/
/*                                                                     */
/*  FILE        :ICS_sample.c                                          */
/*  DATE        :24, Aug, 2017                                         */
/*  DESCRIPTION :Main Program                                          */
/*  CPU TYPE    :RX71M                                                 */
/*                                                                     */
/*  NOTE:THIS IS A TYPICAL EXAMPLE.                                    */
/*                                                                     */
/***********************************************************************/

#ifdef  __ICCRX__
  #include "r_cg_macrodriver.h"
  #include <math.h>
#elif   __CCRX__
  #include <machine.h>
  #include <mathf.h>
  #include <stdint.h>
  #include "iodefine.h"
#else
  //
#endif

#include "ICS2_RX71M.h"

void main(void);

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
/* Set option bytes */

#ifdef   __ICCRX__

  #if __LITTLE_ENDIAN__
    #pragma public_equ="__MDES",0xFFFFFFFFUL
  #else
    #pragma public_equ="__MDES",0xFFFFFFF8UL
  #endif

  #pragma public_equ="__OFS0",0xFFFFFFFFUL
  #pragma public_equ="__OFS1",0xFFFFFFFFUL

#elif   __CCRX__  
  
  
#endif


#define   TWOPI    (2.0f*3.1415926535f)
#define   BRR    0     //  HOCO=16MHz PLL=x15 = CPUCLK=240MHz, BRR=0, ICS=7.5Mbps =ICS CLK=60MHz
                       //  BRR  BPS   DTLclock 
                       //   0  7.500M   60M    
                       //   1  3.750M   30M    
                       //   2  2.500M   20M    
                       //   3  1.875M   15M     
                       //   4  1.500M   12M
#define   MODE   3     // 0:16bit 8ch,one time
                       // 1:32bit 8ch,2 times
                       // 2:32bit 4ch,one time
                       // 3:32bit 12ch 3 times


#ifdef  __ICCRX__
  #pragma location=0x0F000
  __no_init  uint32_t dtc_table[256]; /* alignment 0x000 */
#elif   __CCRX__
  #pragma address dtc_table=0x0F000
  uint32_t dtc_table[256];            /* alignment 0x000 */
#else
#endif

float   wt    = 0.0f;
float   dwt   = TWOPI*50.0f/10000.0f;
float   refu  = 0.0f;
float   refv  = 0.0f;
float   refw  = 0.0f;
float   m     = 1.0f;
float   dm    = 0.0002;
int32_t up    = 0;


void main(void)
{
    SYSTEM.PRCR.WORD = 0xA50B;
    SYSTEM.OSTDCR.BYTE = 0x00;

    SYSTEM.HOCOCR2.BYTE = 0x00;      // HOCO = 16MHz
    SYSTEM.HOCOCR.BYTE  = 0x00;      // HOCO start
    while (SYSTEM.OSCOVFSR.BIT.HCOVF==0) { /* no code */ }  // Wait until HOCO stable
    
    SYSTEM.SCKCR3.WORD = 0x0100;     // Set system clock = HOCO
    SYSTEM.SCKCR.LONG  = 0x21021222; // FCK=1/4, ICK=1/2, BCK=1/4
                                     // PCLKA=1/2 = 120MHz, SCIF, MTU, GPT
                                     // PCLKB=1/4 =  60MHz, others
                                     // PCLKC=1/4 =  60MHz, ADC unit0
                                     // PCLKD=1/4 =  60MHz, ADC unit1

    SYSTEM.PLLCR.BIT.PLIDIV    = 0;  // PLL input is divided by 1
    SYSTEM.PLLCR.BIT.PLLSRCSEL = 1;  // PLL source is HOCO
    SYSTEM.PLLCR.BIT.STC       = 29; // PLL mul rate is x15
    SYSTEM.PLLCR2.BYTE         = 0;  // PLL on
    while (SYSTEM.OSCOVFSR.BIT.PLOVF==0) { /* no code */ } // Wait until PLL stable

    SYSTEM.SCKCR3.WORD = 0x0400;     // Set system clock = PLL (16MHzx15=240MHz)

  //******************************************************************************************************
    ics2_init((void *)(&dtc_table), ICS_SCI0_P32_P33, 6, BRR, MODE);  //CN1  Default      
  //ics2_init((void *)(&dtc_table), ICS_SCI1_P26_P30, 6, BRR, MODE);  //     if E1 is not used.
  //ics2_init((void *)(&dtc_table), ICS_SCI1_P16_P15, 6, BRR, MODE);  //CN2        
  //ics2_init((void *)(&dtc_table), ICS_SCI2_P13_P12, 6, BRR, MODE);  //CN10       
  //ics2_init((void *)(&dtc_table), ICS_SCI2_P50_P52, 6, BRR, MODE);  //           
  //ics2_init((void *)(&dtc_table), ICS_SCI4_PB1_PB0, 6, BRR, MODE);  //           
  //ics2_init((void *)(&dtc_table), ICS_SCI5_PC3_PC2, 6, BRR, MODE);  //       
  //ics2_init((void *)(&dtc_table), ICS_SCI5_PA4_PA3, 6, BRR, MODE);  //           
  //ics2_init((void *)(&dtc_table), ICS_SCI6_P00_P01, 6, BRR, MODE);  //           
  //ics2_init((void *)(&dtc_table), ICS_SCI6_P32_P33, 6, BRR, MODE);  //CN1        
  //ics2_init((void *)(&dtc_table), ICS_SCI6_PB1_PB0, 6, BRR, MODE);  //           
  //ics2_init((void *)(&dtc_table), ICS_SCI7_P90_P92, 6, BRR, MODE);  //       
  //******************************************************************************************************

#ifdef  __ICCRX__
    EI();
#elif   __CCRX__
    set_ipl(0); /* set lowest IPL level */
    setpsw_i();  // enable interrput
#else
    //
#endif

    while (1)
    {
        int32_t i;
        
        wt = wt + dwt;
        if (wt>TWOPI) { wt = wt - TWOPI; }
        
        if (up==1)
        {
             m = m + dm;
             if (m>=1.0f) { up = 0; }
        }
        else
        {
             m = m - dm;
             if (m<=0.5f) { up = 1; }
        }

        refu  = m * sinf(wt);
        refv  = m * sinf(wt-TWOPI*1.0f/3.0f);
        refw  = m * sinf(wt-TWOPI*2.0f/3.0f);

        for(i=0;i<2500;i++) { nop(); }

        ics2_watchpoint();
    }
}
